package com.cneeds.rvhelpers.decoration;

import android.graphics.Canvas;
import android.graphics.Rect;
import android.support.v7.widget.RecyclerView;
import android.view.View;

import com.cneeds.rvhelpers.adapter.PinnedHeaderAdapter;

/**
 * PinnedHeader对应的ItemDecoration，这里只是为了画悬浮标题，实际标题item不是用ItemDecoration实现的，是直接用Adapter里面的ItemView实现
 *
 * 参考博客：https://www.jianshu.com/p/1558cbd20d17
 * https://blog.csdn.net/little762/article/details/79391807
 * https://blog.csdn.net/briblue/article/details/70211942
 *
 * 参考github: https://github.com/tuacy/RecyclerPinnedHeader/blob/master/pinnedheader/src/main/java/com/tuacy/pinnedheader/PinnedHeaderItemDecoration.java
 *
 * author : maiqh
 */
public class PinnedHeaderItemDecoration extends RecyclerView.ItemDecoration {

	/**
	 * 把要固定的View绘制在上层
	 */
	@Override
	public void onDrawOver(Canvas c, RecyclerView parent, RecyclerView.State state) {
		super.onDrawOver(c, parent, state);
		//确保是PinnedHeaderAdapter的adapter,确保有View
		if (parent.getAdapter() instanceof PinnedHeaderAdapter && parent.getChildCount() > 0) {
			PinnedHeaderAdapter adapter = (PinnedHeaderAdapter) parent.getAdapter();

			View firstView = parent.getChildAt(0);
			int firstAdapterPosition = parent.getChildAdapterPosition(firstView);
			int pinnedHeaderPosition = getPinnedHeaderViewPosition(firstAdapterPosition, adapter);

			if (pinnedHeaderPosition != -1) {
				// 生成要固定的头部标题View
				RecyclerView.ViewHolder pinnedHeaderViewHolder = adapter.onCreateViewHolder(parent,
						adapter.getItemViewType(pinnedHeaderPosition));
				adapter.onBindViewHolder(pinnedHeaderViewHolder, pinnedHeaderPosition);
				View pinnedHeaderView = pinnedHeaderViewHolder.itemView;
				// 确定粘性头部的宽高和坐标
				ensurePinnedHeaderViewLayout(pinnedHeaderView, parent);
				// 获取下一个标题与当前固定标题的偏移量
				int sectionPinOffset = 0;
				for (int index = 0; index < parent.getChildCount(); index++) {
					if (adapter.isPinnedPosition(parent.getChildAdapterPosition(parent.getChildAt(index)))) {
						View sectionView = parent.getChildAt(index);
						int sectionTop = sectionView.getTop();
						int pinViewHeight = pinnedHeaderView.getHeight();
						if (sectionTop < pinViewHeight && sectionTop > 0) {
							// sectionPinOffset是个负值
							sectionPinOffset = sectionTop - pinViewHeight;
						}
						break;
					}
				}
				// 重绘当前的粘性头部，实现粘性头部被定上去的效果
				int saveCount = c.save();
				RecyclerView.LayoutParams layoutParams = (RecyclerView.LayoutParams) pinnedHeaderView.getLayoutParams();
				if (layoutParams == null) {
					throw new NullPointerException("PinnedHeaderItemDecoration");
				}
				c.translate(layoutParams.leftMargin, sectionPinOffset);
				c.clipRect(0, 0, parent.getWidth(), pinnedHeaderView.getMeasuredHeight()); // ?
				// Manually render this view (and all of its children) to the given Canvas.
				pinnedHeaderView.draw(c);
				c.restoreToCount(saveCount);
			}
		}
	}

	/**
	 * 要给每个item设置间距主要靠这个函数来实现
	 */
	@Override
	public void getItemOffsets(Rect outRect, View view, RecyclerView parent, RecyclerView.State state) {
	}

	/**
	 * 根据第一个可见的adapter的位置去获取临近的一个要固定的position的位置（就是需要固定的标题在adapter的数据中的位置）
	 *
	 * @param adapterFirstVisible 第一个可见的adapter的位置
	 * @return -1：未找到 >=0 找到位置
	 */
	private int getPinnedHeaderViewPosition(int adapterFirstVisible, PinnedHeaderAdapter adapter) {
		for (int index = adapterFirstVisible; index >= 0; index--) {
			if (adapter.isPinnedPosition(index)) {
				return index;
			}
		}
		return -1;
	}

	private void ensurePinnedHeaderViewLayout(View pinView, RecyclerView recyclerView) {
		if (pinView.isLayoutRequested()) {
			/**
			 * 用的是RecyclerView的宽度测量，和RecyclerView的宽度一样
			 */
			RecyclerView.LayoutParams layoutParams = (RecyclerView.LayoutParams) pinView.getLayoutParams();
			if (layoutParams == null) {
				throw new NullPointerException("PinnedHeaderItemDecoration");
			}
			int widthSpec = View.MeasureSpec.makeMeasureSpec(
				recyclerView.getMeasuredWidth() - layoutParams.leftMargin - layoutParams.rightMargin, View.MeasureSpec.EXACTLY);

			int heightSpec;
			if (layoutParams.height > 0) {
				heightSpec = View.MeasureSpec.makeMeasureSpec(layoutParams.height, View.MeasureSpec.EXACTLY);
			} else {
				heightSpec = View.MeasureSpec.makeMeasureSpec(0, View.MeasureSpec.UNSPECIFIED);
			}
			pinView.measure(widthSpec, heightSpec);
			pinView.layout(0, 0, pinView.getMeasuredWidth(), pinView.getMeasuredHeight());
		}
	}
}
